/**************************************************\
*	WinHLLAPI sample application
*
*	Developed by Jolly Giant Software 1998
*
*	This application demonstrates how to use
*	the C++ language to interface with the
*	HLLAPI functions to access QWS3270 PLUS
*
*	Last modified: Dec. 03/2000
\**************************************************/

//  :)
// This "C++ version" only differs in some prototypes from the C version,
// and is in no way attempting to use OOP
//  :)

#define NOATOM
#define NOGDICAPMASKS
#define NOMETAFILE
#define NOMINMAX
#define NOOPENFILE
#define NORASTEROPS
#define NOSCROLL
#define NOSOUND
#define NOTEXTMETRIC
#define NOWH
#define NOCOMM
#define NOKANJI
#define NOCRYPT
#define NOMCX

#include <windows.h>
#include "Main.h"
#include "Hllapi.h"
#include "resource.h"

// ****************************************
// global variables

// needed for run-time linking to the dll only.  For
// load-time linking, you can remove it
typedef UINT (CALLBACK* LPFNWINHLLAPIFUNC)(LPWORD, LPBYTE, LPWORD, LPWORD);
typedef int (CALLBACK* LPFNWINHLLAPISTARTUPFUNC)(WORD, LPWHLLAPIDATA);

HINSTANCE			hInst;				// handle to current instance
HMODULE				hDLL;				// Handle to DLL
LPFNWINHLLAPIFUNC	lpfnWinHLLAPIFunc;	// Function pointer
LPFNWINHLLAPISTARTUPFUNC lpfnWinHLLAPIStartupFunc;

LPSTR				lpszAppName = APPNAME;
LPSTR				lpszTitle = APPTITLE;
LPSTR				lpDataString;		// pointer to data for dll

/**********************************************************************\
 MAIN WINDOWS FUNCTION
\**********************************************************************/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG		msg;

	hInst = hInstance; // Store instance handle in our global variable
	if (!hPrevInstance)
	{
		// Perform instance initialization:
		if (!InitApplication(hInstance))
		{
			return (FALSE);
		}
	}

	// Perform application initialization:
	if (!InitInstance(hInstance, nCmdShow))
	{
		return (FALSE);
	}

	if (!InitDLL())
	{
		return FALSE;
	}

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	FreeLibrary(hDLL);

	return (msg.wParam);

	lpCmdLine; // This will prevent 'unused formal parameter' warnings
}


/**********************************************************************\
 INITIATE THE APPLICATION
 PURPOSE: CHECKS FOR ANOTHER INSTANCE OF THE APPLICATION, REGISTERS THE
 CLASSES FOR THE MAIN WINDOW AND THE WINDOW FOR DISPLAYING THE
 PRESENTATION SPACE
\**********************************************************************/
BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;
	WNDCLASS  wcDisplay;
    HWND      hwnd;

    hwnd = FindWindow (lpszAppName, lpszTitle);
    if (hwnd)
	{
        // We found another version of ourself
        if (IsIconic(hwnd))
		{
            ShowWindow(hwnd, SW_RESTORE);
        }
        SetForegroundWindow (hwnd);
        return FALSE;
	}

	// Fill in window class structure with parameters that describe
	// the main window.
	wc.style         = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc   = (WNDPROC)WndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;
	wc.hIcon         = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_QWSICON));
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(GetStockObject(WHITE_BRUSH));
	wc.lpszMenuName  = MAKEINTRESOURCE(IDR_MAINMENU);
	wc.lpszClassName = lpszAppName;

	// Register the window class
	if (!RegisterClass(&wc))
		return FALSE;

	// Fill in window class structure with the parameters for the
	// display window
	wcDisplay.style			= CS_HREDRAW | CS_VREDRAW;
	wcDisplay.lpfnWndProc	= (WNDPROC)Display;
	wcDisplay.cbClsExtra	= 0;
	wcDisplay.cbWndExtra	= 0;
	wcDisplay.hInstance		= hInstance;
	wcDisplay.hIcon         = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_QWSICON));
	wcDisplay.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcDisplay.hbrBackground	= (HBRUSH)(GetStockObject(WHITE_BRUSH));
	wcDisplay.lpszMenuName	= NULL;
	wcDisplay.lpszClassName	= "DisplayWindow";

	if (!RegisterClass(&wcDisplay))
		return FALSE;

	return TRUE;
}


/**********************************************************************\
 INITIATE THE INSTANCE
\**********************************************************************/
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;

	hWnd = CreateWindow(
						lpszAppName,
						lpszTitle,
						WS_OVERLAPPEDWINDOW,
						190,
						200,
						450,
						280,
						NULL,
						NULL,
						hInstance,
						NULL);

	if (!hWnd) {
		return (FALSE);
	}

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return (TRUE);
}


/**********************************************************************\
 MAIN WINDOW MESSAGE PROCESSING FUNCTION
\**********************************************************************/
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	int nRet;
	WHLLAPIDATA data;
	WORD wVersion;
	char szText[MAX_PATH];
	char szCaption[25];
	UINT uiType = MB_OK;

	switch (message)
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);

		//Parse the menu selections:
		switch (wmId)
		{
		case IDM_FUNCTION:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_FUNCTIONDLG), hWnd, (DLGPROC)FunctionDlg);
			break;

		case IDM_STARTUP_ONE:
			wVersion = MAKEWORD(1, 3);
			nRet = lpfnWinHLLAPIStartupFunc(wVersion, &data);
			if (nRet != 0)
			{
				wsprintf(szText,
					"Startup Function failed\n\nReason: %d\n"
					"Description: %s\n\n"
					"Requested Version: %d.%d\n"
					"Supported Version: %d.%d",
					nRet, data.szDescription, LOBYTE(wVersion), HIBYTE(wVersion), LOBYTE(data.wVersion), HIBYTE(data.wVersion));
				lstrcpy(szCaption, "WHLLAPI Startup Failure");
				uiType |= MB_ICONSTOP;
			}
			else
			{
				wsprintf(szText,
					"Startup Function successful\n"
					"Description: %s\n\n"
					"Requested Version: %d.%d\n"
					"Supported Version: %d.%d",
					// supplier description
					data.szDescription,
					// requested version
					LOBYTE(wVersion), HIBYTE(wVersion),
					// supported major version
					LOBYTE(data.wVersion),
					// minor version
					HIBYTE(data.wVersion));
				lstrcpy(szCaption, "WHLLAPI Startup Success");
				uiType |= MB_ICONINFORMATION;
			}

			MessageBox(hWnd, szText, szCaption, uiType);

			if (nRet != 0)
			{
				FreeLibrary(hDLL);
				SendMessage(hWnd, WM_COMMAND, (WPARAM)IDM_EXIT, 0);
			}
			break;

		case IDM_STARTUP_TWO:
			wVersion = MAKEWORD(2, 4);
			nRet = lpfnWinHLLAPIStartupFunc(wVersion, &data);
			if (nRet != 0)
			{
				wsprintf(szText,
					"Startup Function failed\n\nReason: %d\n"
					"Description: %s\n\n"
					"Requested Version: %d.%d\n"
					"Supported Version: %d.%d",
					nRet, data.szDescription, LOBYTE(wVersion), HIBYTE(wVersion), LOBYTE(data.wVersion), HIBYTE(data.wVersion));
				lstrcpy(szCaption, "WHLLAPI Startup Failure");
				uiType |= MB_ICONSTOP;
			}
			else
			{
				wsprintf(szText,
					"Startup Function successful\n"
					"Description: %s\n\n"
					"Requested Version: %d.%d\n"
					"Supported Version: %d.%d",
					// supplier description
					data.szDescription,
					// requested version
					LOBYTE(wVersion), HIBYTE(wVersion),
					// supported major version
					LOBYTE(data.wVersion),
					// minor version
					HIBYTE(data.wVersion));
				lstrcpy(szCaption, "WHLLAPI Startup Success");
				uiType |= MB_ICONINFORMATION;
			}

			MessageBox(hWnd, szText, szCaption, uiType);

			if (nRet != 0)
			{
				FreeLibrary(hDLL);
				SendMessage(hWnd, WM_COMMAND, (WPARAM)IDM_EXIT, 0);
			}
			break;

		case IDM_ABOUT:
			DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUT), hWnd, (DLGPROC)About);
			break;

		case IDM_EXIT:
			DestroyWindow (hWnd);
			break;

		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
		}
		break;

	   case WM_DESTROY:
		   PostQuitMessage(0);
		   break;

	   default:
		   return (DefWindowProc(hWnd, message, wParam, lParam));
   }
   return (0);
}


/**********************************************************************\
 MESSAGE PROCESSING FUNCTION FOR THE ABOUT DIALOG BOX
\**********************************************************************/
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		ShowWindow (hDlg, SW_HIDE);

		CenterWindow (hDlg, GetWindow (hDlg, GW_OWNER));

		ShowWindow (hDlg, SW_SHOW);
		return (TRUE);

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK)
		{
            EndDialog(hDlg, TRUE);
            return (TRUE);
		}
		break;

	case WM_CLOSE:
		EndDialog(hDlg, TRUE);
		return TRUE;
	}
    return FALSE;
}


/**********************************************************************\
 MESSAGE PROCESSING FUNCTION FOR THE WINDOW THAT DISPLAYS THE
 PRESENTATION SPACE COPIED FROM THE EMULATOR
\**********************************************************************/
LRESULT CALLBACK Display(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HWND hwndEdit;
	BYTE data[8689];
	RECT rc;
	HFONT hfnt;

	switch (message)
	{
	case WM_CREATE:
		hwndEdit = CreateWindowEx( 0L,
			"Edit",
			NULL,
			ES_MULTILINE
			| ES_UPPERCASE
			| WS_CHILD
			| WS_VSCROLL
			| WS_HSCROLL
			| WS_VISIBLE,
			0, 0, 0, 0,
			hWnd,
			NULL,
			hInst,
			NULL);

		// change the font for this window to a fixed size font for better character alignment
		hfnt = (HFONT)GetStockObject(ANSI_FIXED_FONT);
		SendMessage(hwndEdit, WM_SETFONT ,(WPARAM) hfnt, MAKELPARAM(FALSE, 0));
		BuildString((LPSTR)data);
		SendMessage(hwndEdit, WM_SETTEXT, 0, (LPARAM)data);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_SIZE:
		// always size window to size of parent window client area
		GetClientRect(hWnd, &rc);
		SetWindowPos(hwndEdit, hWnd, rc.left, rc.top, rc.right, rc.bottom, SWP_NOZORDER);
		break;

	default:
		return (DefWindowProc(hWnd, message, wParam, lParam));
	}
	return (0);
}


/**********************************************************************\
 MESSAGE PROCESSING FUNCTION FOR THE DIALOG BOX THAT LETS US CALL
 THE VARIOUS WINHLLAPI TEST FUNCTIONS
\**********************************************************************/
LRESULT CALLBACK FunctionDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	WORD	FuncNumber;
	int index;

	switch (message)
	{
	case WM_INITDIALOG:
		HandleInit(hDlg);
		return (TRUE);

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDEXECUTE:
			// get the selection from the combo box
			FuncNumber = (WORD)SendDlgItemMessage(hDlg, IDC_FUNCTION, CB_GETCURSEL, 0, 0L);
			// call the WinHLLAPI function with the appropriate parameters
			HandleExecute(hDlg, FuncNumber);
            return (TRUE);

		case IDCANCEL:
			EndDialog(hDlg, TRUE);
			return TRUE;
		}

		switch (HIWORD(wParam))
		{
		case CBN_SELCHANGE:
			index = SendDlgItemMessage(hDlg, IDC_FUNCTION, CB_GETCURSEL, 0, 0L);
			HandleSelChange(hDlg, index);
			return TRUE;
		}
		break;

	case WM_CLOSE:
		EndDialog(hDlg, TRUE);
		return TRUE;
	}
    return FALSE;
}


/**********************************************************************\
 PURPOSE: CENTERS CHILD WINDOW INSIDE PARENT WINDOW
\**********************************************************************/
BOOL CenterWindow (HWND hwndChild, HWND hwndParent)
{
	RECT    rChild, rParent, rWorkArea;
	int     wChild, hChild, wParent, hParent;
	int     xNew, yNew;
	BOOL  bResult;

	// Get the Height and Width of the child window
	GetWindowRect (hwndChild, &rChild);
	wChild = rChild.right - rChild.left;
	hChild = rChild.bottom - rChild.top;

	// Get the Height and Width of the parent window
	GetWindowRect (hwndParent, &rParent);
	wParent = rParent.right - rParent.left;
	hParent = rParent.bottom - rParent.top;

	// Get the limits of the 'workarea'
	bResult = SystemParametersInfo(
		SPI_GETWORKAREA,  // system parameter to query or set
		sizeof(RECT),
		&rWorkArea,
		0);
	if (!bResult)
	{
		rWorkArea.left = rWorkArea.top = 0;
		rWorkArea.right = GetSystemMetrics(SM_CXSCREEN);
		rWorkArea.bottom = GetSystemMetrics(SM_CYSCREEN);
	}

	// Calculate new X position, then adjust for workarea
	xNew = rParent.left + ((wParent - wChild) /2);
	if (xNew < rWorkArea.left)
	{
		xNew = rWorkArea.left;
	}
	else if ((xNew+wChild) > rWorkArea.right)
	{
		xNew = rWorkArea.right - wChild;
	}

	// Calculate new Y position, then adjust for workarea
	yNew = rParent.top  + ((hParent - hChild) /2);
	if (yNew < rWorkArea.top)
	{
		yNew = rWorkArea.top;
	}
	else if ((yNew+hChild) > rWorkArea.bottom)
	{
		yNew = rWorkArea.bottom - hChild;
	}

	// Set it, and return
	return SetWindowPos (hwndChild, NULL, xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}


/**********************************************************************\
 PURPOSE: CONVERTS AN INTEGER VALUE TO A CHAR POINTER
\**********************************************************************/
LPSTR ItoLPSTR(LPSTR szBuffer, int i)
{
	wsprintf(szBuffer,"%i", i);

	return szBuffer;
}


/**********************************************************************\
 FUNCTION FOR THE COMBO BOX IN THE DIALOG BOX FOR CHOOSING THE
 WINHLLAPI FUNCTIONS. CHANGES STATIC TEXT, ENABLES AND DISABLES
 TEXT EDIT BOXES, AND CHANGES THE TEXT IN THE EDIT BOXES DEPENDING
 ON THE FUNCTION CHOSEN IN THE COMBO BOX
\**********************************************************************/
void HandleSelChange(HWND hDlg, int index)
{
	switch (index)
	{
	case 0:		// WINHLLAPI FUNCTION #1: connect presentation space
		// set the text for the static text controls
		SendDlgItemMessage(hDlg, IDC_STATIC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Session ID:");
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Data Length:");
		SendDlgItemMessage(hDlg, IDC_STATIC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Return Code:");
		// disable the edit boxes and set the default text for this sample app
		SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, FALSE, 0L);
		SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"A");
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"1");
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Not set");
		//SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_LENGTH, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, EM_SETREADONLY, TRUE, 0L);
		break;
	case 1:		// WINHLLAPI FUNCTION #2: disconnect presentation space
		// set the text for the static text controls
		SendDlgItemMessage(hDlg, IDC_STATIC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Session ID:");
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Data Length:");
		SendDlgItemMessage(hDlg, IDC_STATIC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Return Code:");
		// disable the edit boxes and set the default text for this sample app
		SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, FALSE, 0L);
		SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"A");
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"1");
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Not set");
		//SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_LENGTH, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, EM_SETREADONLY, TRUE, 0L);
		break;
	case 2:		// WINHLLAPI FUNCTION #3: send key
		// set the text for the static text controls
		SendDlgItemMessage(hDlg, IDC_STATIC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"String to send:");
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Data Length:");
		SendDlgItemMessage(hDlg, IDC_STATIC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Return Code:");
		// enable the data edit box, the user has to type the text here to send
		// to the emulator, and the length edit box. Set the default text to
		// the 'ENTER' keystring @E
		SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"@E");
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"2");
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Not set");
		SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, FALSE, 0L);
		SendDlgItemMessage(hDlg, IDC_LENGTH, EM_SETREADONLY, FALSE, 0L);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, EM_SETREADONLY, TRUE, 0L);
		break;
	case 3:		// WINHLLAPI FUNCTION #5: copy presentation space
		// set the text for the static text controls
		SendDlgItemMessage(hDlg, IDC_STATIC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"N/A");
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"N/A");
		SendDlgItemMessage(hDlg, IDC_STATIC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Return Code:");
		// disable the edit boxes and set the default text
		SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"");
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"");
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Not set");
		SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_LENGTH, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, EM_SETREADONLY, TRUE, 0L);
		break;
	case 4:		// WINHLLAPI FUNCTION #6: search presentation space
		// set the text for the static text controls
		SendDlgItemMessage(hDlg, IDC_STATIC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Search string:");
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Data Length:");
		SendDlgItemMessage(hDlg, IDC_STATIC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Search position:");
		// ensable the edit boxes for the data (search string) and the length (number
		// of characters to search for) and the return code (position to search from)
		// Set some default search text
		SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"QWS3270");
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"7");
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"1");
		SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, FALSE, 0L);
		SendDlgItemMessage(hDlg, IDC_LENGTH, EM_SETREADONLY, FALSE, 0L);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, EM_SETREADONLY, FALSE, 0L);
		break;
	case 5:		// WINHLLAPI FUNCTION #7: query cursor location
		// set the text for the static text controls
		SendDlgItemMessage(hDlg, IDC_STATIC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"N/A");
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"N/A");
		SendDlgItemMessage(hDlg, IDC_STATIC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Return Code:");
		// disable the edit boxes and set the default text
		SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"");
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"");
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Not set");
		SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_LENGTH, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, EM_SETREADONLY, TRUE, 0L);
		break;
	case 6:		// WINHLLAPI FUNCTION #15: copy string to presentation space
		// set the text for the static text controls
		SendDlgItemMessage(hDlg, IDC_STATIC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"String to copy");
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Data Length:");
		SendDlgItemMessage(hDlg, IDC_STATIC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Position:");
		// enable the edit boxes for the data (string to copy), the length (number
		// of characters to copy), and the return code (position to copy to)
		SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Sample Text");
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"11");
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"1");
		SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, FALSE, 0L);
		SendDlgItemMessage(hDlg, IDC_LENGTH, EM_SETREADONLY, FALSE, 0L);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, EM_SETREADONLY, FALSE, 0L);
		break;
	case 7:		// WINHLLAPI FUNCTION #40: set cursor position
		// set the text for the static text controls
		SendDlgItemMessage(hDlg, IDC_STATIC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"N/A");
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"N/A");
		SendDlgItemMessage(hDlg, IDC_STATIC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"Cursor position:");
		// enable the return code edit box (location to set the cursor to) and set a default
		// for the new cursor position
		SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"");
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"");
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"1862");
		SendDlgItemMessage(hDlg, IDC_DATA, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_LENGTH, EM_SETREADONLY, TRUE, 0L);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, EM_SETREADONLY, FALSE, 0L);
		break;
	}
}


/**********************************************************************\
 MESSAGE HANDLING FUNCTION TO HANDLE THE INITIALIZATION OF THE
 APPLICATIONS DIALOG BOX. SETS UP THE COMBO BOX CHOICES AND THE
 INITIAL TEXT IN THE TEXT BOXES
\**********************************************************************/
void HandleInit(HWND hDlg)
{
	char *Functions[] = {
		{"Connect Presentation Space"},
		{"Disconnect Presentation Space"},
		{"Send Key"},
		{"Copy Presentation Space"},
		{"Search Presentation Space"},
		{"Query Cursor Location"},
		{"Copy String to Presentation Space"},
		{"Set Cursor Location"}
		};

	int i;
	int iNumStrings = 8;

	ShowWindow (hDlg, SW_HIDE);
	// load the combo box with the possible functions to call
	for (i = 0; i < iNumStrings; i++)
	{
		SendDlgItemMessage(	hDlg,
			IDC_FUNCTION,
			CB_ADDSTRING,
			0,
			(LPARAM) (LPCTSTR) Functions[i]);
	}
	// set the initial text in the edit controls
	SendDlgItemMessage(hDlg, IDC_DATA, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"A");
	SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"1");
	SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)(LPCSTR)"0");
	// select the first item in the combo box
	SendDlgItemMessage(hDlg, IDC_FUNCTION, CB_SETCURSEL, 0, 0L);

	CenterWindow (hDlg, GetWindow (hDlg, GW_OWNER));

	ShowWindow (hDlg, SW_SHOW);
}


/**********************************************************************\
 MESSAGE HANDLING FUNCTION FOR THE EXECUTE BUTTON IN THE DIALOG
 BOX.  CHECKS WHICH FUNCTION TO CALL BASED ON THE SELECTION IN THE
 COMBO BOX, SETS UP PARAMETERS FOR THE FUNCTION CALL, CALLS THE
 WINHLLAPI FUNCTION, CALLS FUNCTIONS TO TIME THE HLLAPI FUNCTION
 CALL, CALLS FUNCTION TO HANDLE THE RETURN VALUES AND CODES
\**********************************************************************/
void HandleExecute(HWND hDlg, WORD FuncNumber)
{
	LPWORD	lpwFuncNumber;
	LPWORD	lpwLength;
	LPWORD	lpwReturnCode;

	WORD	Length;			// storage for data length
	WORD	ReturnCode = 0;		// storage for return code
	char	Data[3841];		// adjust this for the size of the particular presentation space
							// you're dealing with.  In this sample we assume a presentation
							// space size of not more than 1920 characters (times 2 for EAB
							// (extended attribute bytes) + 1 for NULL terminator)
	BOOL	bSuccess;

	lpwFuncNumber = &FuncNumber;
	lpwLength = &Length;
	lpwReturnCode = &ReturnCode;
	lpDataString = Data;

	switch (FuncNumber)
	{
	case 0:		// connect presentation space
		// the data length here is always 1
		Length = 1;
		// get the session id from the dialog edit box
		SendMessage(GetDlgItem(hDlg, IDC_DATA), WM_GETTEXT, 2, (LPARAM)Data);
		//lstrcpy(Data, "A");
		Data[1] = 0;
		// position is not needed for this function call
		// adjust the function number
		FuncNumber = 1;
		break;
	case 1:		// disconnect presentation space
		// the data length here is always 1
		Length = 1;
		// get the session id from the dialog edit box
		SendMessage(GetDlgItem(hDlg, IDC_DATA), WM_GETTEXT, 2, (LPARAM)Data);
		//lstrcpy(Data, "A");
		Data[1] = 0;
		// position is not needed for this function call
		FuncNumber = 2;
		break;
	case 2:		// send key
		// first we'll get the length, if any is specified
		Length = (WORD)GetDlgItemInt(hDlg, IDC_LENGTH, &bSuccess, FALSE);
		if (!bSuccess)
			// set to default value
			Length = (WORD)SendDlgItemMessage(hDlg, IDC_DATA, WM_GETTEXT, 256, (LPARAM)Data);
		else
			SendDlgItemMessage(hDlg, IDC_DATA, WM_GETTEXT, 256, (LPARAM)Data);
		// check to make sure we have a valid length and valid text
		if (strlen(Data) <= 0)
		{
			MessageBox (hDlg, "Please enter a string to send", "Empty String", MB_OK | MB_ICONSTOP);
			return;
		}
		if ((Length <= 0) || (Length > 255))
			Length = 255;
		// position is not needed for this function call
		FuncNumber = 3;
		break;
	case 3:		// copy presentation space
		// we don't have to send data or anything else to the DLL
		lstrcpy(Data, "");
		Length = 0;
		FuncNumber = 5;
		break;
	case 4:		// search presentation space
		// first we'll get the number of characters if specified
		Length = (WORD)GetDlgItemInt(hDlg, IDC_LENGTH, &bSuccess, FALSE);
		if (!bSuccess)
			Length = (WORD)SendDlgItemMessage(hDlg, IDC_DATA, WM_GETTEXT, 1921, (LPARAM)Data);
		else
			SendDlgItemMessage(hDlg, IDC_DATA, WM_GETTEXT, 1921, (LPARAM)Data);
		if (strlen(Data) < 1)
		{
			MessageBox (hDlg, "Please enter a string to search for", "Error", MB_OK | MB_ICONSTOP);
			return;
		}
		if ((Length < 1) || (Length > 1920))
			Length = 1920;
		// get the position to search from
		ReturnCode = (WORD)GetDlgItemInt(hDlg, IDC_RETURNCODE, &bSuccess, (LPARAM)Data);
		if ((!bSuccess) || (ReturnCode < 1) || (ReturnCode > 1920))
			ReturnCode = 1;	// set to default value
		FuncNumber = 6;
		break;
	case 5:		// query cursor location
		// no values needed for this function
		lstrcpy(Data, "");
		Length = 0;
		ReturnCode = 0;
		FuncNumber = 7;
		break;
	case 6:		// copy string to presentation space
		// get the length if specified
		Length = (WORD)GetDlgItemInt(hDlg, IDC_LENGTH, &bSuccess, FALSE);
		if (!bSuccess)
			Length = (WORD)SendDlgItemMessage(hDlg, IDC_DATA, WM_GETTEXT, 1921, (LPARAM)Data);
		else
			SendDlgItemMessage(hDlg, IDC_DATA, WM_GETTEXT, 1921, (LPARAM)Data);
		if (strlen(Data) < 1)
		{
			MessageBox (hDlg, "Please enter a string to copy", "Error", MB_OK | MB_ICONSTOP);
			return;
		}
		if ((Length < 1) || (Length > 1920))
			Length = 1920;
		ReturnCode = (WORD)GetDlgItemInt(hDlg, IDC_RETURNCODE, &bSuccess, FALSE);
		if ((!bSuccess) || (ReturnCode < 1) || (ReturnCode > 1920))
			ReturnCode = 1;	// set to default value
		FuncNumber = 15;
		break;
	case 7:		// set cursor location
		// all we need here is the position for the cursor, stored in ReturnCode
		lstrcpy(Data, "");
		Length = 0;
		ReturnCode = (WORD)GetDlgItemInt(hDlg, IDC_RETURNCODE, &bSuccess, FALSE);
		if ((!bSuccess) || (ReturnCode < 1) || (ReturnCode > 1920))
			ReturnCode = 1862;
		FuncNumber = 40;
		break;
	}
	TimeIt(hDlg, TRUE);
	//	function call to hllapi dll
	// NOTE: Use this one if you are linking to the dll at runtime
	lpfnWinHLLAPIFunc (lpwFuncNumber, (LPBYTE)lpDataString, lpwLength, lpwReturnCode);
	//		 and this one if you are linking at loadtime
//	WinHLLAPI (lpwFuncNumber, lpDataString, lpwLength, lpwReturnCode);
	TimeIt(hDlg, FALSE);
	HandleReturn(hDlg, lpwFuncNumber, lpwLength, lpwReturnCode);
}


/**********************************************************************\
 PURPOSE: TIME THE HLLAPI FUNCTION CALL, CONVERT THE DATA TO A
 FORMAT THAT CAN BE DISPLAYED IN A TEXT BOX, DISPLAY THE TIME
\**********************************************************************/
void TimeIt(HWND hDlg, BOOL Start)
{
	static DWORD dwStartTime;
	DWORD dwEndTime;
	char szBuf[10];

	if (Start)
	{
		// get the time and store it
		dwStartTime = GetTickCount();
	}
	else
	{
		// get the time
		dwEndTime = GetTickCount();
		// subtract it from the start time and display the result
		wsprintf(szBuf, "%d", dwEndTime - dwStartTime);
		strcat(szBuf, " ms");
		SendDlgItemMessage (hDlg, IDC_TIME, WM_SETTEXT, 0, (LPARAM)szBuf);
	}
}


/**********************************************************************\
 PURPOSE: HANDLE THE RETURN VALUES AND ERROR CODES FROM THE HLLAPI
 FUNCTION CALLS.  DISPLAY VALUES AND CODES IN DIALOG BOX
\**********************************************************************/
void HandleReturn(HWND hDlg, LPWORD lpwFuncNumber, LPWORD lpwLength, LPWORD lpwReturnCode)
{
	char szBuf[10];

	switch (*lpwFuncNumber)
	{
	case 1:	// connect presentation space
	case 2:	// disconnect presentation space
	case 3:	// send key
		// display the return code
		wsprintf(szBuf, "%i", *lpwReturnCode);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)szBuf);
		DecodeMessage(*lpwReturnCode);
		break;
	case 5:	// copy presentation space
		// display the return code
		wsprintf(szBuf, "%i", *lpwReturnCode);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)szBuf);
		DecodeMessage(*lpwReturnCode);
		if (*lpwReturnCode == WHLLOK)
			DisplayPresentationSpace();
		break;
	case 6:	// search presentation space
		// display the return code
		wsprintf(szBuf, "%i", *lpwReturnCode);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)szBuf);
		if (*lpwReturnCode == WHLLOK)
		{
			wsprintf(szBuf, "%i", *lpwLength);
			SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)szBuf);
			if (*lpwLength == 0)	// not found
				SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)"String not found");
			else
				SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)"Found at position");
		}
		DecodeMessage(*lpwReturnCode);
		break;
	case 7:	// query cursor location
		wsprintf(szBuf, "%i", *lpwLength);
		SendDlgItemMessage(hDlg, IDC_LENGTH, WM_SETTEXT, 0, (LPARAM)szBuf);
		SendDlgItemMessage(hDlg, IDC_STATIC_LENGTH, WM_SETTEXT, 0, (LPARAM)"Cursor Position:");
		wsprintf(szBuf, "%i", *lpwReturnCode);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)szBuf);
		DecodeMessage(*lpwReturnCode);
		break;
	case 15:// copy string to presentation space
		wsprintf(szBuf, "%i", *lpwReturnCode);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)szBuf);
		DecodeMessage(*lpwReturnCode);
		break;
	case 40:// set cursor location
		wsprintf(szBuf, "%i", *lpwReturnCode);
		SendDlgItemMessage(hDlg, IDC_RETURNCODE, WM_SETTEXT, 0, (LPARAM)szBuf);
		DecodeMessage(*lpwReturnCode);
		break;
	}
}


/**********************************************************************\
 PURPOSE: TRANSLATE THE HLLAPI ERROR CODE INTO A MESSAGE, DISPLAY
 THE MESSAGE IN A MESSAGE BOX IF THE ERROR CODE IS NOT ZERO
\**********************************************************************/
void DecodeMessage(WORD Code)
{
	char szBuf[40];
	char szMessage[256];

	switch (Code)
	{
		// parse error codes
	case WHLLOK:
		return;
	case WHLLNOTCONNECTED:
		lstrcpy (szBuf, "NOT CONNECTED TO PRESENTATION SPACE");
		break;
	case WHLLPARAMETERERROR:
		lstrcpy (szBuf, "PARAMETER ERROR/INVALID FUNCTION");
		break;
	case WHLLFTXCOMPLETE:
		lstrcpy (szBuf, "FILE TRANSFER COMPLETE");
		break;
	case WHLLBUSY:
		lstrcpy (szBuf, "PRESENATION SPACE IS BUSY");
		break;
	case WHLLINHIBITED:
		lstrcpy (szBuf, "INHIBITED / KKEYBOARD LOCKED");
		break;
	case WHLLTRUNCATED:
		lstrcpy (szBuf, "DATA TRUNCATED");
		break;
	case WHLLPOSITIONERROR:
		lstrcpy (szBuf, "INVALID PRESENTATION SPACE POSITION");
		break;
	case WHLLNOTAVAILABLE:
		lstrcpy (szBuf, "UNAVAILABLE OPERATION");
		break;
	case WHLLSYSERROR:
		lstrcpy (szBuf, "SYSTEM ERROR");
		break;
	case WHLLNOTSUPPORTED:
		lstrcpy (szBuf, "FUNCTION NOT SUPPORTED");
		break;
	case WHLLUNAVAILABLE:
		lstrcpy (szBuf, "RESOURCE IS UNAVAILABLE");
		break;
	case WHLLPSENDED:
		lstrcpy (szBuf, "THE SESSION WAS STOPPED");
		break;
	case WHLLUNDEFINEDKEY:
		lstrcpy (szBuf, "UNDEFINED KEY COMBINATION");
		break;
	case WHLLOIAUPDATE:
		lstrcpy (szBuf, "OIA UPDATED");
		break;
	case WHLLPSUPDATE:
		lstrcpy (szBuf, "PS UPDATED");
		break;
	case WHLLBOTHUPDATE:
		lstrcpy (szBuf, "BOTH PS AND OIA UPDATED");
		break;
	case WHLLNOFIELD:
		lstrcpy (szBuf, "NO SUCH FIELD FOUND");
		break;
	case WHLLNOKEYSTROKES:
		lstrcpy (szBuf, "NO KEYSTROKES ARE AVAILABLE");
		break;
	case WHLLPSCHANGED:
		lstrcpy (szBuf, "PS OR OIA CHANGED");
		break;
	case WHLLFTXABORTED:
		lstrcpy (szBuf, "FILE TRANSFER ABORTED");
		break;
	case WHLLZEROLENFIELD:
		lstrcpy (szBuf, "FIELD LENGTH IS ZERO");
		break;
	case WHLLKEYOVERFLOW:
		lstrcpy (szBuf, "KEYSTROKE OVERFLOW");
		break;
	case WHLLSFACONN:
		lstrcpy (szBuf, "OTHER APPLICATION ALREADY CONNECTED");
		break;
	case WHLLTRANCANCLI:
		lstrcpy (szBuf, "MESSAGE SENT INBOUND TO HOST CANCELLED");
		break;
	case WHLLTRANCANCL:
		lstrcpy (szBuf, "OUTBOUND TRANS FROM HOST CANCELLED");
		break;
	case WHLLHOSTCLOST:
		lstrcpy (szBuf, "CONTACT WITH HOST WAS LOST");
		break;
	case WHLLOKDISABLED:
		lstrcpy (szBuf, "THE FUNCTION WAS SUCCESSFUL");
		break;
	case WHLLNOTCOMPLETE:
		lstrcpy (szBuf, "THE REQUESTED FN WAS NOT COMPLETED");
		break;
	case WHLLSFDDM:
		lstrcpy (szBuf, "ONE DDM SESSION ALREADY CONNECTED");
		break;
	case WHLLSFDPEND:
		lstrcpy (szBuf, "DISCONNECTED W ASYNC REQUESTS PENDING");
		break;
	case WHLLBUFFINUSE:
		lstrcpy (szBuf, "SPECIFIED BUFFER CURRENTLY IN USE");
		break;
	case WHLLNOMATCH:
		lstrcpy (szBuf, "NO MATCHING REQUEST FOUND");
		break;
	case WHLLLOCKERROR:
		lstrcpy (szBuf, "API ALREADY LOCKED OR UNLOCKED");
		break;
	case WHLLINVALIDFUNCTIONNUM:
		lstrcpy (szBuf, "INVALID FUNCTION NUMBER");
		break;
	case WHLLFILENOTFOUND:
		lstrcpy (szBuf, "FILE NOT FOUND");
		break;
	case WHLLACCESSDENIED:
		lstrcpy (szBuf, "ACCESS DENIED");
		break;
	case WHLLMEMORY:
		lstrcpy (szBuf, "INSUFFICIENT MEMORY");
		break;
	case WHLLINVALIDENVIRONMENT:
		lstrcpy (szBuf, "INVALID ENVIRONMENT");
		break;
	case WHLLINVALIDFORMAT:
		lstrcpy (szBuf, "INVALID FORMAT");
		break;
	case WHLLINVALIDPSID:
		lstrcpy (szBuf, "INVALID PRESENTATION SPACE ID");
		break;
	case WHLLINVALIDRC:
		lstrcpy (szBuf, "INVALID ROW OR COLUMN CODE");
		break;
	}
	strcpy(szMessage, "The Function returned an error code!\n\n");
	strcat(szMessage, "The Error code translated as:\n");
	strcat(szMessage, szBuf);
	MessageBox (NULL, szMessage, "Error Code" , MB_OK | MB_ICONINFORMATION);
}


/**********************************************************************\
 CREATES THE WINDOW FOR DISPLAYING THE PRESENTATION SPACE COPIED
 IN HLLAPI FUNCTION 5
\**********************************************************************/
void DisplayPresentationSpace()
{
	HWND hWnd;

	hWnd = CreateWindow(
						"DisplayWindow",
						"Copy Presentation Space",
						WS_OVERLAPPEDWINDOW,
						90,
						100,
						620,
						380,
						NULL,
						NULL,
						hInst,
						NULL);

	if (!hWnd)
	{
		MessageBox(NULL, "Window not created!", NULL, MB_OK | MB_ICONSTOP);
		exit (1);
	}

	ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);
}


/**********************************************************************\
 PURPOSE: CONVERT THE STRING RETURNED FROM THE EMULATOR TO A FORMAT
 THAT CAN BE DISPLAYED IN THE EDIT BOX IN THE DISPLAY WINDOW, ADD
 LINE FEEDS AND BLANKS AT APPROPRIATE PLACES
\**********************************************************************/
void BuildString(LPSTR data)
{
	UINT	i;
	UINT	c;
	char tmp[3];
	strcpy(data, "");

	for (i = 1; i <= 3840; i++)
	{
		c = __toascii(*lpDataString++);
		wsprintf(tmp, "%x", c);
		if (strlen(tmp) < 2)
		{
			strncat(data, "0", 1);
			strncat(data, tmp, 1);

		}
		else
			// must be 2 characters
			strncat(data, tmp, 2);
		if (i%80 == 0)
			strncat(data, "\r\n", 2);
		else if (i%4 == 0)
			strncat(data, " ", 1);
	}
}


/**********************************************************************\
 PURPOSE: INITIALIZES THE DLL, DISPLAYS MESSAGE TO USER IF DLL WAS NOT
 FOUND, OR THE DLL WAS FOUND BUT THE ENTRY POINT WAS NOT, AND RETURNS
 FALSE TO THE CALLING FUNCTION.  ELSE RETURNS TRUE
\**********************************************************************/
BOOL InitDLL(void)
{
	char msg[256];

	// intitialize the WinHLLAPI dll
	hDLL = LoadLibrary("whllapi");
	if (hDLL != NULL)
	{
		lpfnWinHLLAPIFunc = (LPFNWINHLLAPIFUNC)GetProcAddress(hDLL,
			"WinHLLAPI");
		if (!lpfnWinHLLAPIFunc)
		{
			// notify user
			MessageBox (NULL,
				"DLL Entry Point 'WinHLLAPI' not found",
				"Entry Point not found",
				MB_OK | MB_ICONSTOP);
			// handle the error
			FreeLibrary(hDLL);
			return (FALSE);
		}
		lpfnWinHLLAPIStartupFunc = (LPFNWINHLLAPISTARTUPFUNC)GetProcAddress(hDLL,
			"WinHLLAPIStartup");
		if (!lpfnWinHLLAPIStartupFunc)
		{
			MessageBox (NULL,
				"DLL Entry Point 'WinHLLAPIStartup' not found",
				"Entry Point not found",
				MB_OK | MB_ICONSTOP);
			// handle the error
			FreeLibrary(hDLL);
			return (FALSE);
		}
	}
	else
	{
		strcpy (msg, "Failure loading WINHLLAPI DLL\n\n");
		strcat (msg, "Make sure the file whllapi.dll is in your windows\n");
		strcat (msg, "directory or system directory if you are running\n");
		strcat (msg, "Windows95 or Windows98, or in your system32 directory\n");
		strcat (msg, "for Windows NT");
		MessageBox (NULL, msg, "Fatal: DLL not found", MB_OK | MB_ICONSTOP);
		return (FALSE);
	}
	return (TRUE);
}